from gym.envs.doom.doom_env import DoomEnv, MetaDoomEnv
from gym.envs.doom.doom_basic import DoomBasicEnv
from gym.envs.doom.doom_corridor import DoomCorridorEnv
from gym.envs.doom.doom_defend_center import DoomDefendCenterEnv
from gym.envs.doom.doom_defend_line import DoomDefendLineEnv
from gym.envs.doom.doom_health_gathering import DoomHealthGatheringEnv
from gym.envs.doom.doom_my_way_home import DoomMyWayHomeEnv
from gym.envs.doom.doom_predict_position import DoomPredictPositionEnv
from gym.envs.doom.doom_take_cover import DoomTakeCoverEnv
from gym.envs.doom.doom_deathmatch import DoomDeathmatchEnv
